#include "mgos.h"
#include "mgos_i2c.h"
#include "mgos_sht31.h"
#include "mgos_si7021.h"
#include <fcntl.h>
#include <sys/ioctl.h>

#define I2CBUSNR 7

void i2c_scanner(struct mgos_i2c *i2c) {
  int i;
  if (!i2c) {
    LOG(LL_ERROR, ("No global I2C bus configured"));
    return;
  }

  for(i=0x3; i<0x77; i++) {
    bool ret;
    ret=mgos_i2c_read(i2c, i, NULL, 0, true);
    if (ret)
      LOG(LL_INFO, ("I2C Address 0x%02x %s", i, ret?"true":"false"));
  }
}

bool i2c_dumpregs(struct mgos_i2c *i2c, uint8_t i2caddr) {
  uint16_t reg;
  int value;

  for(reg=0; reg<256; reg++) {
    value=mgos_i2c_read_reg_b(i2c, i2caddr, reg);
    if (value<0) {
      printf(" XX");
    } else {
      printf(" %02x", value);
    }
    if (reg%16==15) printf("\n");
  }
  printf("\n");
  return true;
}

bool do_sht31(struct mgos_sht31 *sensor) {
  float temp, humid;

  if (!sensor) return false;

  temp=mgos_sht31_getTemperature(sensor);
  humid=mgos_sht31_getHumidity(sensor);
  LOG(LL_INFO, ("temperature=%.2fC humidity=%.1f%%", temp, humid));

  return true;
}

bool do_si7021(struct mgos_si7021 *sensor) {
  float temp, humid;

  if (!sensor) return false;

  temp=mgos_si7021_getTemperature(sensor);
  humid=mgos_si7021_getHumidity(sensor);
  LOG(LL_INFO, ("temperature=%.2fC humidity=%.1f%%", temp, humid));

  return true;
}

int main() {
  struct mgos_i2c *i2c;
  struct mgos_si7021 *si7021;
  struct mgos_sht31 *sht31;

  if (!mgos_i2c_open(I2CBUSNR)) {
    LOG(LL_ERROR, ("Cannot open I2C bus %u", I2CBUSNR));
    return -1;
  }
  if (!(i2c = mgos_i2c_get_global())) {
    LOG(LL_ERROR, ("Cannot open I2C bus"));
    return -2;
  }

  i2c_scanner(i2c);

  if (!(sht31 = mgos_sht31_create(i2c, 0x44))) {
    LOG(LL_ERROR, ("Cannot create SHT31 device"));
  }

  if (!(si7021 = mgos_si7021_create(i2c, 0x40))) {
    LOG(LL_ERROR, ("Cannot create SI7021 device"));
  }

  for (;;) {
    do_sht31(sht31);
    do_si7021(si7021);
    sleep(5);
  }

  mgos_sht31_destroy(&sht31);
  mgos_si7021_destroy(&si7021);

  return 0;
}
